%%%%%%%%%%%%%
% THR_EST.M %
%%%%%%%%%%%%%

% written by:

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu
% http://www.ssc.wisc.edu/~bhansen/


% This is a Matlab procedure.  It computes estimates and confidence
% intervals for threshold models.  The procedure takes the form

% qhat = thr_est(dat,names,yi,xi,qi,h)

% The inputs are:
%   dat     = data matrix (nxk)
%   names   = variable names (kx1), corresponding to dat matrix
%   yi      = index of dependent (y) variable, e.g.: yi = 1
%   xi      = indexes of independent (x) variables, e.g.: xi = 2|3
%   qi      = index of threshold (q) variable, e.g.: qi = 4;
%   h       = heteroskedasticity indicator
%             Set h=0 to impose homoskedasticity assumption
%             Set h=1 to use White-correction for heteroskedasticity

% Output:
%   qhat    = LS estimate of threshold

% The remaining outputs are printed to the screen.

% Notes:  
%   (1)  Do not include a constant in the independent variables;
%        the program automatically adds an intercept to the regression.

%   (2)  There are three other control parameters, governing the choice
%        of confidence level, the nonparametric method used to compute
%        the nuisance parameter in the event of heteroskedastic, and 
%        whether to print the graph of the likelihood.  These controls
%        are listed at the beginning of the procedure code.




% Example:
% If the nxk matrix "dat" contains the dependent variable in the first
% column, the independent variables in the second through tenth columns,
% and the threshold variable in the fifth.  If the error is homoskedastic:

%     xi = [2 3 4 5 6 7 8 9 10];
%     qhat = thr_est(dat,1,xi,5,0);

% while if the error is (possibly)heteroskedatic, 
% replace the second line with

%     qhat = thr_est(dat,1,xi,5,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function qhat=thr_est(dat,names,yi,xi,qi,h)

% Control Parameters, can be modified if desired %
conf1_=.95; % Confidence Level for Confidence Regions %
conf2_=.8;  % Confidence Level for first step of two-step%
            % Confidence Regions for regression parameters %
nonpar_=2;  % Indicator for non-parametric method used to
            % estimate nuisance scale in the presence of
            % heteroskedasticity (only relevant if h=1).
            % Set nonpar_=1  to estimate regressions using
            % a quadratic.
            % Set nonpar_=2  to estimate regressions using
            % an Epanechnikov kernel with automatic bandwidth. @
graph_=1;   % Set graph_=1 for the program to produce the graph
            % of the concentrated likelihood in gamma.
            % Set graph_=0 to not view the graph.

if (1-(h==0))*(1-(h==1))
    fprintf('You have entered h = %f\n',h);
    disp('This number must be either 0 (homoskedastic case) or 1 (heteoskedastic)');
    disp('The program will either crash or produce invalid results');
end;
if (1-(nonpar_==1))*(1-(nonpar_==2))*(h==1)
    fprintf('You have entered _nonpar = %f\n',nonpar_);
    disp('This number should be either 1 (quadratic regression)');
    disp('or 2 (kernel regression)');
    disp('The program will employ the quadratic regression method');
    disp(' ');
    disp(' ');
end;

n=length(dat(:,1));
q=dat(:,qi);

% Sorted index of q %
qs=0;
for i=1:length(q)
    for i_=1:length(q)
        temp=0;
        for j=1:length(q)
            if (q(i_)>q(j))|(i_>=j&q(i_)==q(j))
                temp=temp+1;
            end;
        end;
        if temp==i
            qs=[qs;i_];
    end;
    end;
end;
qs=qs(2:length(qs));
q=q(qs);

y=dat(qs,yi);
x=[ones(n,1),dat(qs,xi)];
k=length(x(1,:));
yname=names(yi,:);
qname=names(qi,:);
xname=['Constant';names(xi,:)];

mi=inv(x'*x);
beta=mi*(x'*y);
e=y-x*beta;
ee=e'*e;
sig=ee/(n-k);
xe=x.*(e*ones(1,length(x(1,:))));
if h==0
    se=sqrt(diag(mi)*sig);
else
    se=sqrt(diag(mi*xe'*xe*mi));
end;
vy=sum((y-mean(y)').*(y-mean(y)'))';
r_2=1-ee/vy;

qs=unique(q);
qn=length(qs);
sn=zeros(qn,1);

irb=zeros(n,1);
mm=zeros(k,k);
sume=zeros(k,1);
ci=0;

r=1;
while r<=qn
    irf=(q<=qs(r));
    ir=irf-irb;
    irb=irf;
    ci=ci+sum(ir)';
    xir=x(1,:);
    temp=0;
    for i=1:length(ir)
        if ir(i)==1
            if temp==0
                xir=x(i,:);
                temp=1;
            else
                xir=[xir;x(i,:)];
            end;
        end;
    end;
    mm=mm+xir'*xir;
    sume=sume+(ir'*xe)';
    mmi=mm-mm*mi*mm;
    if (ci>k+1)*(ci<(n-k-1))
        sn(r)=ee-sume'*(inv(mmi)*sume);
    else
        sn(r)=ee;
    end;
    r=r+1;
end;

for i=1:length(sn) %minindc%
    if sn(i)==min(sn)
        rmin=i;
        break;
    end;
end;
smin=sn(rmin);
qhat=qs(rmin);
sighat=smin/n;

i1=(q<=qhat);
i2=1-i1;

x1=0;y1=0;x2=0;y2=0;ind1=0;ind2=0;
for i=1:length(i1)
    if i1(i)==1
        if ind1==0
            x1=x(i,:);
            y1=y(i,:);
            ind1=1;
        else
            x1=[x1;x(i,:)];
            y1=[y1;y(i,:)];
        end;
    else
        if ind2==0
            x2=x(i,:);
            y2=y(i,:);
            ind2=1;
        else
            x2=[x2;x(i,:)];
            y2=[y2;y(i,:)];
        end;
    end; 
end;
mi1=inv(x1'*x1);
mi2=inv(x2'*x2);
beta1=mi1*(x1'*y1);
beta2=mi2*(x2'*y2);
e1=y1-x1*beta1;
e2=y2-x2*beta2;
ej=[e1;e2];
n1=length(y1(:,1));
n2=length(y2(:,1));
ee1=e1'*e1;
ee2=e2'*e2;
sig1=ee1/(n1-k);
sig2=ee2/(n2-k);
sig_jt=(ee1+ee2)/(n-2*k);
if h==0
    se1=sqrt(diag(mi1)*sig_jt);
    se2=sqrt(diag(mi2)*sig_jt);
else
    se1=sqrt(diag(mi1*(x1.*(e1*(ones(1,length(x1(1,:))))))'*(x1.*(e1*(ones(1,length(x1(1,:))))))*mi1));
    se2=sqrt(diag(mi2*(x2.*(e2*(ones(1,length(x2(1,:))))))'*(x2.*(e2*(ones(1,length(x2(1,:))))))*mi2));
end;
vy1=sum((y1-mean(y1)').*(y1-mean(y1)'))';
vy2=sum((y2-mean(y2)').*(y2-mean(y2)'))';
r2_1=1-ee1/vy1;
r2_2=1-ee2/vy2;
r2_joint=1-(ee1+ee2)/vy;

if h==0;
    lr=(sn-smin)/sighat;
else
    r1=(x*(beta1-beta2)).*(x*(beta1-beta2));
    r2=r1.*((ej.*ej)*ones(1,length(r1(1,:))));
    qx=[q.^0,q.^1,q.^2];
    qh=[qhat.^0,qhat.^1,qhat.^2];
    m1=(r1'/qx')';
    m2=(r2'/qx')';
    g1=qh*m1;
    g2=qh*m2;
    if nonpar_==2;
        sigq=sqrt(mean((q-mean(q)').^2));
        hband=2.344*sigq/(n^(.2));
        u=(qhat-q)/hband;
        u2=u.^2;
        f=mean((1-u2).*(u2<=1))'*(1.5/(hband^2));
        df=-mean((1-u2).*(u2<=1))'*(1.5/(hband^2));
        eps=r1-qx*m1;
        sige=(eps'*eps)/(n-3);
        hband=sige/(4*f*((m1(3)+(m1(2)+2*m1(3)*qhat)*df/f)^2));
        u2=((qhat-q)/hband).^2;
        kh=((1-u2)*.75/hband).*(u2<=1);
        g1=mean(kh.*(r1*ones(1,length(kh(1,:)))))';
        g2=mean(kh.*(r2*ones(1,length(kh(1,:)))))';
    end;
    eta2=g2/g1;
    lr=(sn-smin)/eta2;
end;
c1=-2*log(1-sqrt(conf1_));
c2=-2*log(1-sqrt(conf2_));
lr1=(lr>=c1);
lr2=(lr>=c2);
if max(lr1)==1
    minind_lr1=0;
    for i=1:length(lr1)
        if lr1(i)==min(lr1)
            minind_lr1=i;
            break;
        end;
    end;
    qhat1=qs(minind_lr1);
    minind_lr1v=0;
    for i=1:length(lr1)
        if lr1(qn+1-i)==min(lr1)
            minind_lr1v=i;
            break;
        end;
    end;
    qhat2=qs(qn+1-minind_lr1v);
else
    qhat1=qs(1);
    qhat2=qs(qn);
end;

temp=(normcdf([.01:.01:3]')*2-1)>=conf1_;
for i=1:length(temp)
    if temp(i)==max(temp)
        z=i/100;
        break;
    end;
end;
beta1l=beta1-se1*z;
beta1u=beta1+se1*z;
beta2l=beta2-se2*z;
beta2u=beta2+se2*z;
r=1;
while r<=qn;
    if lr2(r)==0
        i1=(q<=qs(r));
        x1=0;y1=0;ind1=0;
        for i=1:length(i1)
            if i1(i)==1
                if ind1==0
                    x1=x(i,:);
                    y1=y(i,:);
                    ind1=1;
                else
                    x1=[x1;x(i,:)];
                    y1=[y1;y(i,:)];
                end;
            end;
        end;
        if det(x1'*x1)>0
            mi1=inv(x1'*x1);
            b1=mi1*(x1'*y1);
            e1=y1-x1*b1;
            if h==0
                ser1=sqrt(diag(mi1)*(e1'*e1)/(length(y1(:,1))-k));
            else
                ser1=sqrt(diag(mi1*(x1.*(e1*ones(1,length(x1(1,:)))))'*(x1.*(e1*ones(1,length(x1(1,:)))))*mi1));
            end;
            beta1l=min([beta1l';(b1-ser1*z)'])';
            beta1u=max([beta1u';(b1+ser1*z)'])';
        end;
        i2=1-i1;
        x2=0;y2=0;ind2=0;
        for i=1:length(i2)
            if i2(i)==1
                if ind2==0
                    x2=x(i,:);
                    y2=y(i,:);
                    ind2=1;
                else
                    x2=[x2;x(i,:)];
                    y2=[y2;y(i,:)];
                end;
            end;
        end;
        if det(x2'*x2)>0
            mi2=inv(x2'*x2);
            b2=mi2*(x2'*y2);
            e2=y2-x2*b2;
            if h==0
                ser2=sqrt(diag(mi2)*(e2'*e2)/(length(y2(:,1))-k));
            else
                ser2=sqrt(diag(mi2*(x2.*(e2*ones(1,length(x2(1,:)))))'*(x2.*(e2*ones(1,length(x2(1,:)))))*mi2));
            end;
            beta2l=min([beta2l';(b2-ser2*z)'])';
            beta2u=max([beta2u';(b2+ser2*z)'])';
        end;
    end;
    r=r+1;
end;

disp('Global OLS Estimation, Without Threshold');
disp('----------------------------------------');
fprintf('Dependent Variable:     ');
fprintf(yname);
fprintf('\n');
if h==1
    disp('Heteroskedasticity Correction Used');
elseif h==0
    disp('OLS Standard Errors Reported');
end;
disp('');
disp('Variable    Estimate      St Error');
disp('----------------------------------');
for i=1:length(beta)
    fprintf(xname(i,:));
    fprintf('      %f      %f\n',beta(i),se(i));
end;
disp(' ');
fprintf('Observations:                     %u\n',n);
fprintf('Degrees of Freedom:               %u\n',n-k);
fprintf('Sum of Squared Errors:            %f\n',ee);
fprintf('Residual Variance:                %f\n',sig);
fprintf('R-squared:                        %f\n',r_2);
fprintf('Heteroskedasticity Test (P-Value):%f\n',het_test(ej,x));
disp(' ');
disp(' ');
disp('Threshold Estimation');
disp('--------------------');
disp(' ');
fprintf('Threshold Variable =  ');
fprintf(qname(1,:));
fprintf('\n');
fprintf('Threshold Estimate:               %f\n',qhat);
fprintf('.%u Confidence Interval:       [%f,%f]\n',conf1_*100,qhat1,qhat2);
fprintf('Sum of Squared Errors             %f\n',ee1+ee2);
fprintf('Residual Variance:                %f\n',sig_jt);
fprintf('Joint R-Squared:                  %f\n',r2_joint);
fprintf('Heteroskedasticity Test (p-value):%f\n',het_test(ej,x));
disp(' ');
disp(' ');
disp('-------------------------------');
fprintf('Regime1  ');
fprintf(qname(1,:));
fprintf('<=%f\n',qhat);
disp('');
disp('Parameter Estimates');
disp('');
disp('Variable    Estimate      St Error');
disp('----------------------------------');
for i=1:length(beta)
    fprintf(xname(i,:));
    fprintf('      %f      %f\n',beta1(i),se1(i));
end;
disp('');
fprintf('.%u Confidence Regions for Parameters.\n',conf1_*100);
disp('Variable         Low          High');
disp('----------------------------------');
for i=1:length(beta)
    fprintf(xname(i,:));
    fprintf('      %f      %f\n',beta1l(i),beta1u(i));
end;

disp(' ');
fprintf('Observations:                     %u\n',n1);
fprintf('Degrees of Freedom:               %u\n',n1-k);
fprintf('Sum of Squared Errors:            %f\n',ee1);
fprintf('Residual Variance:                %f\n',sig1);
fprintf('R-squared:                        %f\n',r2_1);
disp(' ');
disp(' ');

fprintf('Regime2  ');
fprintf(qname(1,:));
fprintf('>%f\n',qhat);
disp('');
disp('Parameter Estimates');
disp('');
disp('Variable    Estimate      St Error');
disp('----------------------------------');
for i=1:length(beta)
    fprintf(xname(i,:));
    fprintf('      %f      %f\n',beta2(i),se2(i));
end;
disp('');
fprintf('.%u Confidence Regions for Parameters.\n',conf1_*100);
disp('Variable         Low          High');
disp('----------------------------------');
for i=1:length(beta)
    fprintf(xname(i,:));
    fprintf('      %f      %f\n',beta2l(i),beta2u(i));
end;

disp(' ');
fprintf('Observations:                     %u\n',n2);
fprintf('Degrees of Freedom:               %u\n',n2-k);
fprintf('Sum of Squared Errors:            %f\n',ee2);
fprintf('Residual Variance:                %f\n',sig2);
fprintf('R-squared:                        %f\n',r2_2);


% Graph %
if graph_==1
    clr=ones(qn,1)*c1;
    fig=figure;
    plot(qs,lr,qs,clr);
    legend('LRN(Gema)',strcat(num2str(conf1_*100),'% Critical'));
    title('Confidence Interval Construction for Threshold');
    xlabel(strcat('Threshold Variable:',qname(1,:)));
    ylabel('Likelihood Ratio Sequence in Gema');
    grid on;    
end;

%LATEX%
ttex=fopen('out.tex','a+');
fprintf(ttex,'\\underline{\\large{Global OLS Estimation, Without Threshold}}\\\\\n');
fprintf(ttex,'\\textbf{Dependent Variable:}\n');
fprintf(ttex,yname);
fprintf(ttex,'\\\\\n');
fprintf(ttex,'\\vspace{.1cm}\\\\\n');

if h==1
   fprintf(ttex,'\\small{Heteroskedasticity Correction Used}\\\\\n');
elseif h==0
   Fprintf(ttex,'\\small{OLS Standard Errors Reported}\\\\\n');
end;


fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
fprintf(ttex,'\\hline\n');
fprintf(ttex,'Variable & Estimate & St Error\\\\\n');
fprintf(ttex,'\\hline\\hline\n');
for i=1:length(beta)
    fprintf(ttex,xname(i,:));
    fprintf(ttex,'& %f & %f\\\\\n',beta(i),se(i));
end;
fprintf(ttex,'\\hline\n');
fprintf(ttex,'\\end{tabular}\\\\\n');
fprintf(ttex,'\\vspace{.2cm}\\\\\n');
fprintf(ttex,'\\textbf{Obsevations:}%u\\\\\n',n2);
fprintf(ttex,'\\textbf{Degrees of Freedom:}%u\\\\\n',n2-k);
fprintf(ttex,'\\textbf{Sum of Squared Errors:}%f\\\\\n',ee2);
fprintf(ttex,'\\textbf{Residual Variance:}%f\\\\\n',sig2);
fprintf(ttex,'\\textbf{R-squared:}%f\\\\\n',r2_2);  

fprintf(ttex,'\\vspace{.2cm}\\\\\n');
fprintf(ttex,'\\underline{\\large{Threshold Estimation}}\\\\\n');
fprintf(ttex,'\\vspace{.1cm}\\\\\n');
fprintf(ttex,'\\textbf{Threshold Variable:}');
fprintf(ttex,qname(1,:));
fprintf(ttex,'\\\\\n');
fprintf(ttex,'\\textbf{Threshold Estimate:}%f\\\\\n',qhat);
fprintf(ttex,'\\textbf{.%u Confidence Interval:}[%f,%f]\\\\\n',conf1_*100,qhat1,qhat2);
fprintf(ttex,'\\textbf{Sum of Squared Errors:}%f\\\\\n',ee1+ee2);
fprintf(ttex,'\\textbf{Residual Variance:}%f\\\\\n',sig_jt);
fprintf(ttex,'\\textbf{Joint R-Squared:}%f\\\\\n',r2_joint);
fprintf(ttex,'\\textbf{Heteroskedasticity Test (p-value):}%f\\\\\n',het_test(ej,x));
fprintf(ttex,'\\vspace{.2cm}\\\\\n');

fprintf(ttex,'\\underline{Regime1:  ');
fprintf(ttex,qname(1,:));
fprintf(ttex,'$\\leq$%f}\\\\\n',qhat);
fprintf(ttex,'\\vspace{.1cm}\\\\\n');
fprintf(ttex,'\\underline{Parameter Estimates}\\\\\n');

fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
fprintf(ttex,'\\hline\n');
fprintf(ttex,'Variable & Estimate & St Error\\\\\n');
fprintf(ttex,'\\hline\\hline\n');
for i=1:length(beta)
    fprintf(ttex,xname(i,:));
    fprintf(ttex,'& %f & %f\\\\\n',beta1(i),se1(i));
end;
fprintf(ttex,'\\hline\n');
fprintf(ttex,'\\end{tabular}\\\\\n');
fprintf(ttex,'\\vspace{.2cm}\\\\\n');



fprintf(ttex,'\\underline{.%u Confidence Regions for Parameters:}\\\\\n',conf1_*100);
fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
fprintf(ttex,'\\hline\n');
fprintf(ttex,'Variable & Low & High\\\\\n');
fprintf(ttex,'\\hline\\hline\n');
for i=1:length(beta)
    fprintf(ttex,xname(i,:));
    fprintf(ttex,'& %f & %f\\\\\n',beta1l(i),se1(i));
end;
fprintf(ttex,'\\hline\n');
fprintf(ttex,'\\end{tabular}\\\\\n');
fprintf(ttex,'\\vspace{.1cm}\\\\\n');

fprintf(ttex,'\\textbf{Observations:}%u\\\\\n',n1);
fprintf(ttex,'\\textbf{Degrees of Freedom:}%u\\\\\n',n1-k);
fprintf(ttex,'\\textbf{Sum of Squared Errors:}%f\\\\\n',ee1);
fprintf(ttex,'\\textbf{Residual Variance:}%f\\\\\n',sig1);
fprintf(ttex,'\\textbf{R-squared:}%f\\\\\n',r2_1);
fprintf(ttex,'\\vspace{.2cm}\\\\\n');


fprintf(ttex,'\\underline{Regime2:  ');
fprintf(ttex,qname(1,:));
fprintf(ttex,'$>$%f}\\\\\n',qhat);
fprintf(ttex,'\\vspace{.1cm}\\\\\n');
fprintf(ttex,'\\underline{Parameter Estimates}\\\\\n');

fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
fprintf(ttex,'\\hline\n');
fprintf(ttex,'Variable & Estimate & St Error\\\\\n');
fprintf(ttex,'\\hline\\hline\n');
for i=1:length(beta)
    fprintf(ttex,xname(i,:));
    fprintf(ttex,'& %f & %f\\\\\n',beta2(i),se2(i));
end;
fprintf(ttex,'\\hline\n');
fprintf(ttex,'\\end{tabular}\\\\\n');
fprintf(ttex,'\\vspace{.2cm}\\\\\n');



fprintf(ttex,'\\underline{.%u Confidence Regions for Parameters:}\\\\\n',conf2_*100);
fprintf(ttex,'\\begin{tabular}{|l |l |l |}\n');
fprintf(ttex,'\\hline\n');
fprintf(ttex,'Variable & Low & High\\\\\n');
fprintf(ttex,'\\hline\\hline\n');
for i=1:length(beta)
    fprintf(ttex,xname(i,:));
    fprintf(ttex,'& %f & %f\\\\\n',beta2l(i),se2(i));
end;
fprintf(ttex,'\\hline\n');
fprintf(ttex,'\\end{tabular}\\\\\n');
fprintf(ttex,'\\vspace{.1cm}\\\\\n');

fprintf(ttex,'\\textbf{Observations:}%u\\\\\n',n2);
fprintf(ttex,'\\textbf{Degrees of Freedom:}%u\\\\\n',n2-k);
fprintf(ttex,'\\textbf{Sum of Squared Errors:}%f\\\\\n',ee2);
fprintf(ttex,'\\textbf{Residual Variance:}%f\\\\\n',sig2);
fprintf(ttex,'\\textbf{R-squared:}%f\\\\\n',r2_2);

fclose(ttex);
